# 
# Janosik - Wieloplatformowa odpowiednik programu Patnik 
#
# Copyright (C) 2002, 2003 Janosik Development Team
# 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

"""Modu udostpniajcy API do tworzenia interfejsw i nie tylko.

	Funkcje te umozliwiaj wygodne, mam nadziej, programowanie
	UI bez potrzeby wchodzenia w nisze warstwy implementacji.
	Dodanie te tego poziomu abstrakcji umoliwia te zmiany
	poniej bez wprowadzania wielu zmian w innych miejscach,
	a jedynie raz w API."""


import TMPL
import KXML
import copy
import weryfikacja
import os.path
import os
import misc
import wlasciwosci
import druk


#tmpl:
tmpl = TMPL.TMPL()

def nowy_dokument(typ):
	"""Stworzenie nowego dokumentu ZUS.
	
	Funkcja tworzca nowy dokument na podstawie podanego typu.
	Typ mona uzyska albo wybierajc jeden z dostpnych przy pomocy
	funkcji "lista_dostepnych_typow()":#lista_dostepnych_typow 
	albo np. pobierajc typ istniejcego 
	dokumentu za pomoc funcji "typ_dokumentu()":#typ_dokumentu."""
	
	return [tmpl.nowy_dokument(typ), None, None]

def lista_dostepnych_typow():
	"""Fukcja zwracajca list dostpnych typw.

	Lista dostpnych typw dokumentw ZUS, czyli takich
	dla ktrych istnieje plik tmpl."""
	
	return tmpl.lista_dostepnych()


#dokument:
wlasc = wlasciwosci.wlasciwosci(tmpl)

def typ_dokumentu(dokument):
	"""Funkcja zwracajca typ dokumentu."""
	
	typ = dokument[0][1]['name']
	if typ[:7] != 'platnik':
		typ = 'zus' + typ
	return typ

def lista_sekcji(dokument):
	"""Funkcja zwracajca list sekcji z dokumentu.
	
	Sekcja jest to logiczna grupa pl z dokumentw ZUS.
	Moe ona grupowa pola odnoszce si np. do danych patnika, 
	danych ubezpieczonego, itp.
	Z wyjtkiem sekcji wielokrotnych, wszystkie sekcje wystpuj
	tylko jeden raz w dokumencie."""
	
	return dokument[0][2]

def lista_pol(sekcja):
	"""Funkcja zwracajca list pl z sekcji."""
	
	return sekcja[2]

def wartosc_pola(pole):
	"""Funkcja zwracajca warto podanego pola."""
	
	return pole[2][0]

def zmien_wartosc_pola(pole, wartosc):
	"""Funkcja zmieniajca warto podanego pola."""

	pole[2][0] = wartosc

def atrybut(typ_dokum, nazwa_pola_lub_sekcji, nazwa_atrybutu):
	"""Funkcja zwracajca warto atrybutu pola lub sekcji.
	
	Pola i sekcje posiadaj swoje atrybuty, ktre s niezalene
	od danych w nich zawartych - s wsplne dla wszystkich dokumentw
	danego typu. Moe by to np. opis pola czy sekcji, dugo pola, 
	oznacznie czy dana sekcja jest wielokrotna."""
	
	return wlasc.atrybut(typ_dokum, nazwa_pola_lub_sekcji, nazwa_atrybutu)

def nazwa(pole_lub_sekcja):
	"""Funkcja zwracajca nazw pola lub sekcji.
	
	Nazwa ta moe by przydatna chociaby do pobrania wartoci atrybutu."""
	
	return pole_lub_sekcja[1]['name']

def _add(slownik, nazwa, wartosc):
	if nazwa in slownik:
		slownik[nazwa].append(wartosc)
	else:
		slownik[nazwa] = [wartosc]

def przygotuj_znajdz(dokument):
	"""Funkcja przygotowujca sownik, umoliwiajcy szybkie wyszukiwanie.
	
	W celu wyszukania pola o danej nazwie mona przejrze list pl
	danego dokumentu. Jeeli jednak operacja ta ma by powtrzona kilka
	razy, bardziej opaca si stworzy specjaln struktur umoliwiajc
	szybkie wyszukanie pola. Funkcja ta tworzy i zwraca taki sownik,
	ktry powinien byc podany jako argument do funkcji 
	"znajdz()":#znajdz."""
	
	slownik = {}
	f = lambda t, s = slownik: _add(s, t[1]['name'], t)
	map(f, dokument[0][2])
	for section in dokument[0][2]:
		map(f, section[2])
	return slownik

def znajdz(slownik, nazwa):
	"""Funkcja zwracajca list pl o podanej nazwie.
	
	W celu wyszukania pola o danej nazwie mona przejrze list pl
	danego dokumentu. Jeeli jednak operacja ta ma by powtrzona kilka
	razy, bardziej opaca si stworzy specjaln struktur umoliwiajc
	szybkie wyszukanie pola. Struktur t jest sownik zwracany przez 
	funkcj "przygotuj_znajdz()":#przygotuj_znajdz. 
	Ta funkcja na postawie tego sownika i nazwy pliku zwraca list pl.
	List, a nie pojedyncze pole, poniewa dla dokumentw w ktrym
	pojawia si sekcja wielokrotna, pola z tej sekcji mog si powtarza."""
	
	return slownik[nazwa]

def _nazwa_sekcji(dokument, pozycja):
	return nazwa(dokument[0][2][pozycja])

def dodaj_sekcje(dokument, sekcja):
	"""Funkcja wstawiajca kopie podanej sekcji do dokumentu.

	Dla dokumentw w ktrych wystpuj sekcje wielokrotne,
	zachodzi czasami potrzeba dodania kolejnej takiej sekcji.
	Funkcja ta kopiuje podan sekcje wielokrotn i wstawia 
	ja na koniec listy takich sekcji w dokumencie.
	Zwracany jest te odnosnik do nowo utworzonej sekcji."""
	
	multiple = atrybut(typ_dokumentu(dokument), nazwa(sekcja), 'multiple')
	if multiple == 'TRUE':
		nzw = nazwa(sekcja)
		j = dokument[0][2].index(sekcja) + 1
		l = len(dokument[0][2])
		while j < l and _nazwa_sekcji(dokument, j) == nzw:
			j += 1
		dokument[0][2].insert(j, copy.deepcopy(sekcja))
		for pole in dokument[0][2][j][2]:
			pole[2][0] = ' ' * len(pole[2][0])
		return dokument[0][2][j]
	return None

def usun_sekcje(dokument, sekcja):
	"""Funkcja usuwajca sekcje z dokumentu.
	
	Dla dokumentw w ktrych wystpuj sekcje wielokrotne,
	zachodzi czasami potrzeba usunicia takiej sekcji.
	Funkcja ta usuwa podan sekcje wielokrotn i jeeli
	bya to ostatnia sekcja tego typu, to wstawiana jest
	nowa sekcja i zwracany odnonik do niej. W przeciwnym
	wypadku zwracany jest None."""
	
	multiple = atrybut(typ_dokumentu(dokument), nazwa(sekcja), 'multiple')
	nowa = None
	if multiple == 'TRUE':
		nzw = nazwa(sekcja)
		j = dokument[0][2].index(sekcja)
		l = len(dokument[0][2]) - 1
		pierwsza = j == 0 or _nazwa_sekcji(dokument, j - 1) != nzw
		ostatnia = j == l or _nazwa_sekcji(dokument, j + 1) != nzw
		if pierwsza and ostatnia:
			dokument[0][2].insert(j + 1, copy.deepcopy(sekcja))
			for pole in dokument[0][2][j + 1][2]:
				pole[2][0] = ' ' * len(pole[2][0])
			nowa = dokument[0][2][j + 1]
		dokument[0][2][j:j+1] = []
	return nowa

def weryfikuj(dokument):
	raise NotImplementedError()
	
#	weryfikacja.weryfikuj_pola(dokument[0])

def wylicz_automatycznie(dokument):
	raise NotImplementedError()


#zestaw:
root = os.path.abspath('../dane')

def nowy_platnik(dokument):
	"""Funkcja tworzca nowego patnika.
	
	Nowy patnik tworzoy jest na postawie wypenionego dokumentu
	opisujcego nowego patnika. Dokumentem tego typu jest np.
	platnik_fizyczna.tmpl."""
	
	slownik = przygotuj_znajdz(dokument)
	pole = znajdz(slownik, 'dopl_symbol_platnika')[0]
	nazwa = wartosc_pola(pole).rstrip()
	katalog = os.path.join(root, nazwa)
	os.makedirs(katalog)
	plik = KXML.nowy_dokument(katalog + '.kxml')
	KXML.dodaj_pola(dokument[0], plik)
	KXML.zamknij_dokument(plik)
	return katalog

def nowy_zestaw(nazwa, platnik):
	"""Funkcja tworzca nowy zestaw dokumantw.
	
	Dla podanego patnika tworzony jest nowy zestaw dokumentw
	o podanej nazwie."""
	
	katalog = os.path.join(platnik, nazwa)
	if not os.path.exists(katalog):
		os.makedirs(katalog)
	return katalog

def _znajdz_zestaw(dokument, zestaw):
	katalog = zestaw
	rodzaj = misc.rodzaj_dokumentu(typ_dokumentu(dokument))
	plik = rodzaj + '.kxml'

	if rodzaj == 'Rozliczeniowe':
		# Zakadam e zawsze ostatnim polem w dowolnym dokumencie jest 
		# data wystawienia i e zajmuje ona 8 znakw. Jest to w tym 
		# momencie prawda, ale by moe kiedy to si zmieni...
		data = dokument[0][2][-1][2][-1][2][0]
		plik = '%s %s.kxml' % (data[4:], data[2:4])
		katalog = os.path.join(katalog, rodzaj)

	return os.path.join(katalog, plik)

def dodaj_dokument(dokument, zestaw):
	"""Funkcja dodajca nowy dokument do zestawu."""
	
	zestaw_rodzic = _znajdz_zestaw(dokument, zestaw)
	if not os.path.exists(zestaw_rodzic):
		katalog = os.path.dirname(zestaw_rodzic)
		if not os.path.exists(katalog):
			os.makedirs(katalog)
		plik = KXML.nowy_dokument(zestaw_rodzic)
		KXML.dodaj_pola(dokument[0], plik)
		KXML.zamknij_dokument(plik)
		dokument[1] = zestaw_rodzic
		dokument[2] = 0
	else:
		dokumenty = KXML.wczytaj(zestaw_rodzic)
		dokumenty[2].append(dokument[0])
		KXML.zapisz(dokumenty, zestaw_rodzic)
		dokument[1] = zestaw_rodzic
		dokument[2] = len(dokumenty[2]) - 2

def zapisz_dokument(dokument):
	"""Funkcja zapisujca zmieniony dokument do zestawu."""
	
	zestaw = KXML.wczytaj(dokument[1])
	zestaw[2][dokument[2]] = dokument[0]
	KXML.zapisz(zestaw, dokument[1])
	
def lista_zestawow(platnik):
	"""Funkcja zwracace list zestaww dla podanego patnika."""
	
	try:
		zestawy = map(lambda d, p = platnik: os.path.join(p, d), 
				os.listdir(platnik))
	except OSError:
		return []
	# Paskudny trik, aby pomin katalogi CVS...
	return filter(lambda s: s[-3:] != 'CVS', zestawy)
			
def lista_podzestawow(zestaw, podzestaw):
	"""Funkcja zwracajca list podzestaww.
	
	Dokumenty w zestawie s umieszczone w swego rodzaju drzewie,
	ktrego korzeniem jest zestaw, limi dokumenty, a wzami
	wewntrznymi podzestawy. Aby uzyska list podzestaww
	wywodzcych si z zestawu, naley jako argumenty poda zestaw
	i '' (pusty acuch). Aby uzyska podzestwy poniej, naley jako 
	argumenty poda zestaw i uzyskany za pomoc tej funkcji podzestaw.
	W tym momencie zestaw dzieli sie na dokumenty zgoszeniowe
	patnika, zgoszeniowe ubezpieczonego i rozliczeniowe.
	Za te ostatnie dziel si wedug miesica utworzenia."""
	
	zestaw_rodzic = os.path.join(zestaw, podzestaw)
	try:
		zestawy = map(lambda d, p = podzestaw: os.path.join(p, d), 
				os.listdir(zestaw_rodzic))
	except OSError:
		return []
	return filter(lambda s: s[-3:] != 'CVS', zestawy)

def poziom_wyzej(podzestaw):
	"""Funkcja zwracajca podzestaw bdcy rodzicem danego podzestawu.
	
	Po szczegowe informacj o podzestawach zobacz 
	"lista_podzestawow()":#lista_podzestawow"""
	
	return os.path.dirname(podzestaw)
				

def lista_dokumentow(zestaw, podzestaw):
	"""Funkcja zwracajca list dokumentw w danym podzestawie.
	
	Po szczegowe informacj o podzestawach zobacz 
	"lista_podzestawow()":#lista_podzestawow"""
	
	zestaw_rodzic = os.path.join(zestaw, podzestaw)
	try:
		dokumenty = KXML.wczytaj(zestaw_rodzic)
	except IOError:
		return []
	return map(lambda dok, i, z = zestaw_rodzic: [dok, z, i], dokumenty[2], 
			range(len(dokumenty[2])))
	

def lista_firm():
	"""Funkcja zwracajca list patnikw."""
	
	return lista_zestawow(root)

def nazwa_zestawu(zestaw):
	"""Funckja zwracajca nazw podanego zestawu."""
	
	return os.path.basename(zestaw)

def dla_wszystkich_dokumentow(zestaw, funkcja):
	"""Wykonanie podanaj funkcji dla wszystkich dokumentw w zestawie."""
	
	misc.walktree(zestaw, lambda zestaw, f = funkcja: map(zestaw[2], f))


#druk

def drukuj_dokument(dokument, plik):
	"""Funkcja drukujca dokument do pliku.
	
	Podany dokument jest drukowany do plikw o podanej nazwie,
	z dodanym numerem strony i rozszezeniem.
	Zwracana jest liczba wydrukowanych stron - stworzonych plikw."""

	return druk.drukuj(dokument, plik)
