#!/usr/bin/env python
# 
# Janosik - Wieloplatformowa odpowiednik programu Patnik 
#
# Copyright (C) 2002, 2003 Janosik Development Team
# 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

"""Import plikw kedu do formatu kxml.

	Modu umoliwia import danych z formatu Patnika
	do formatu Janosika."""

from KEDU import KEDUParser
import KXML
import os.path
import TMPL
import os
import sys
import misc
import copy

def importuj_sekcje(section, nazwa, wartosc):
#	print nazwa
	poz = 0
	for pole in section[2]:
#		print pole[1]['name']
		rozm = int(pole[1]['kedu_length'])
#		rozm = len(pole[2][0])
#		print len(wartosc), poz + rozm
		assert len(wartosc) >= poz + rozm
		pole[2][:] = [wartosc[poz:poz + rozm]]
		poz = poz + rozm
	assert len(wartosc) == poz

def _nazwa_pliku(podkatalog):
	numer = 1
	sciezka = os.path.join(podkatalog, '%d.kxml')
	while os.access(sciezka % numer, os.F_OK):
		numer += 1
	return sciezka % numer


def utworz_wielokrotne(pola, nazwy_sekcji):
	sections = pola[2]
	j = 0
	l = len(nazwy_sekcji)
	for i in range(len(sections)):
		assert j <= l
		print sections[j][1]['name'], nazwy_sekcji[j], j
		assert sections[j][1]['name'] == nazwy_sekcji[j]
		
		if 'multiple' in sections[j][1] and sections[j][1]['multiple'] == 'TRUE':
			while j + 1 < l	and nazwy_sekcji[j] == nazwy_sekcji[j + 1]:
				pola[2].insert(j, copy.deepcopy(sections[j]))
				j += 1
		j += 1

	assert j == l
	assert len(pola[2]) == l
				
def importuj_pola(typ, sekcje, plik, tmpl):
	pola = tmpl.nowy_dokument(typ)
	utworz_wielokrotne(pola, map(lambda tab: tab[0], sekcje))
	sections = pola[2]
	for i in range(len(sections)):
		importuj_sekcje(sections[i], sekcje[i][0], sekcje[i][1])
	KXML.dodaj_pola(pola, plik)

	

def importuj_dokument(typ, sekcje, katalog, tmpl, otwarte):
	rodzaj = misc.rodzaj_dokumentu(typ)
	plik = rodzaj + '.kxml'
	print typ

	if rodzaj == 'Rozliczeniowe':
		# Zakadam e zawsze ostatnim polem w dowolnym dokumencie jest 
		# data wystawienia i e zajmuje ona 8 znakw. Jest to w tym 
		# momencie prawda, ale by moe kiedy to si zmieni...
		data = sekcje[-1][-1][-8:]
		plik = '%s %s.kxml' % (data[4:], data[2:4])
		katalog = os.path.join(katalog, rodzaj)

	if not os.path.exists(katalog):
		os.makedirs(katalog)

	if plik not in otwarte:
		otwarte[plik] = KXML.nowy_dokument(os.path.join(katalog, plik))

	importuj_pola(typ, sekcje, otwarte[plik], tmpl)
		
def importuj(infile, dir):
	tmpl = TMPL.TMPL()
	
	katalog = os.path.join(dir, infile)
	try:
		misc.walktree(katalog, os.unlink)
	except OSError:
		pass
	try:
		misc.walktree2(katalog, os.rmdir)
		os.rmdir(katalog)
	except OSError:
		pass
	os.makedirs(katalog)

	parser = KEDUParser(infile, tmpl.lista_dostepnych())
	dokumenty = parser.go()
	otwarte = {}
	for dokument in dokumenty:
		importuj_dokument(dokument[0], dokument[1], katalog, tmpl, otwarte)
	for plik in otwarte:
		KXML.zamknij_dokument(otwarte[plik])

	print "Utworzono %d nowych dokumentw" % len(dokumenty)

def uzycie(nazwa):
	print 'Skadnia: %s plik_wejsciowy katalog_wyjsciowy' % nazwa

def main():
	if not len(sys.argv) == 3:
		uzycie(sys.argv[0])
		sys.exit(2)
	
	importuj(sys.argv[1], sys.argv[2])


if __name__ == "__main__":
	main()
